// Ported to ILI9341_due by GHLawrence2000
//
// https://github.com/ghlawrence2000/ILI9341_due_Buttons
//
// http://forum.arduino.cc/index.php?action=profile;u=164638
//
/*
  ILI9341_due_Buttons.h - Add-on Library for UTFT: Buttons
  Copyright (C)2015 Rinky-Dink Electronics, Henning Karlsen. All right reserved
  
  This library adds simple but easy to use buttons to extend the use
  of the UTFT and URTouch libraries.

  You can find the latest version of the library at 
  http://www.RinkyDinkElectronics.com/

  This library is free software; you can redistribute it and/or
  modify it under the terms of the CC BY-NC-SA 3.0 license.
  Please see the included documents for further information.

  Commercial use of this library requires you to buy a license that
  will allow commercial use. This includes using the library,
  modified or not, as a tool to sell products.

  The license applies to all part of the library including the 
  examples and tools supplied with the library.
*/

#ifndef ILI9341_due_Buttons_h
#define ILI9341_due_Buttons_h

#define ILI9341_due_Buttons_VERSION	10206

#if defined(__AVR__)
	#include "Arduino.h"
	#include "hardware/avr/HW_AVR_defines.h"
#elif defined(__PIC32MX__)
	#include "WProgram.h"
	#include "hardware/pic32/HW_PIC32_defines.h"
#elif defined(__arm__)
	#include "Arduino.h" // This will include energia.h where appropriate
	#include "hardware/arm/HW_ARM_defines.h"
#endif

#include <ILI9341_due.h>
#include <URTouch.h>

#define MAX_BUTTONS	20	// Maximum number of buttons available at one time

// Define presets for button status
#define BUTTON_DISABLED			0x0001
#define BUTTON_SYMBOL			0x0002
#define BUTTON_SYMBOL_REP_3X	0x0004
#define BUTTON_BITMAP			0x0008	
#define BUTTON_NO_BORDER		0x0010
#define BUTTON_UNUSED			0x8000

typedef struct
{
  uint16_t			pos_x, pos_y, width, height;
  uint16_t			flags;
  char				*label;
  bitmapdatatype	data;
} button_type;

class ILI9341_due_Buttons
{
	public:
		ILI9341_due_Buttons(ILI9341_due *ptrILI9341, URTouch *ptrURTouch);

		int		addButton(uint16_t x, uint16_t y, uint16_t width, uint16_t height, char *label, uint16_t flags=0);
		int		addButton(uint16_t x, uint16_t y, uint16_t width, uint16_t height, bitmapdatatype data, uint16_t flags=0);
		void	drawButtons();
		void	drawButton(int buttonID);
		void	enableButton(int buttonID, boolean redraw=false);
		void	disableButton(int buttonID, boolean redraw=false);
		void	relabelButton(int buttonID, char *label, boolean redraw=false);
		boolean	buttonEnabled(int buttonID);
		void	deleteButton(int buttonID);
		void	deleteAllButtons();
		int		checkButtons();
		void	setTextFont(gTextFont font);
		void	setSymbolFont(gTextFont font);
		void	setButtonColors(word atxt, word iatxt, word brd, word brdhi, word back);
		
	protected:
		ILI9341_due *_ILI9341;
		URTouch		*_URTouch;
		button_type	buttons[MAX_BUTTONS];
		word		_color_text, _color_text_inactive, _color_background, _color_border, _color_hilite;
		const uint8_t		*_font_text, *_font_symbol;
};

#endif
